{-------------------------------------------------------------------
*    TOverloadIndicator v1.0                                       *
*                                                                  *
*    Alex Prudkiy                                                  *
*    e-mail: prudkiy@mail.ru                                       *
*    First release: 18.08.1999                                     *
*                                                                  *
*    The idea of this component based on XOSView program from Unix *
*                                                                  *
--------------------------------------------------------------------
*  Any bug reports or suggestions are welcome.                     *
--------------------------------------------------------------------
*  This component is a FREEWARE. You may modify it or do whatever  *
*  You want, but please send me a copy via e-mail.                 *
*                                                                  *
*  The author accepts no responsibility for its use or misuse.     *
*                                                                  *
*  The author assumes no liability for damages, direct or          *
*  consequential, which may result from the use of this software.  *
*                                                                  *
-------------------------------------------------------------------}

unit OverloadIndicator;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, extctrls;
type
   TOverloadIndicatorSpeed = 1..200;
   TTickWidth = 1..MaxInt;
   TNeedDataEvent = procedure(Sender: TObject; var Position: Integer) of object;
type
   TOverloadIndicator = class(TCustomPanel)
   private
      FBackgrnd: TColor;
      FAutoQueryData: Boolean;
      FMaxValue: Integer;
      FSpeedTPS: TOverloadIndicatorSpeed;
      FHistoryCapacity: Integer;
      FHistory: Variant;
      FHistoryPointer: Integer;
      FMinValue: Integer;
      FPosition: Integer;
      FBoxWidth: TTickWidth;
      FOnDataNeed: TNeedDataEvent;
      FTimer: TTimer;
      FLastWidth, FLastBevelWidth, FLastBorderWidth: Integer;
      FRandomShowOnDesign: Boolean;
      FForegroundColor: TColor;

      procedure ChangeBackgrndColor(const Value: TColor);
      procedure SetActive(const Value: Boolean);
      procedure SetHistoryCapacity(const Value: Integer);
      procedure SetMaxValue(const Value: Integer);
      procedure SetMinValue(const Value: Integer);
      procedure SetOnDataNeed(const Value: TNeedDataEvent);
      procedure SetPosition(const Value: Integer);
      procedure OnTimer(Sender: TObject);
      procedure SetSpeedTPS(const Value: TOverloadIndicatorSpeed);
      procedure NewEvent(NewPosition: Integer);
      procedure RecalcBoxWidth;
      procedure HistoryRepaint;

      procedure SetRandomShowOnDesign(const Value: Boolean);
      procedure SetForegroundColor(const Value: TColor);
      procedure SetBoxWidth(const Value: TTickWidth);
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Paint; override;
      procedure RePaint; override;
      procedure ClearHistory;

   published
      { Published declarations }

      // Key properties

      property BackgroundColor: TColor read FBackgrnd write ChangeBackgrndColor default clWhite;
      property ForegroundColor: TColor read FForegroundColor write SetForegroundColor;

      {
      Next three properties used for automatic retreive Position, if
        AutoQueryData=True then OnDataNeed Executed at SpeedTPS per second
      }
      property AutoQueryData: Boolean read FAutoQueryData write SetActive;
      property SpeedTPS: TOverloadIndicatorSpeed read FSpeedTPS write SetSpeedTPS;
      property OnDataNeed: TNeedDataEvent read FOnDataNeed write SetOnDataNeed;

      {
       setting with AutoQueryData=False used for Add new Tick on Intdocator
       }
      property Position: Integer read FPosition write SetPosition;

      {
      Quantity of Ticks on the Indicator (for perfect view must be less Width )
      }
      property HistoryCapacity: Integer read FHistoryCapacity write SetHistoryCapacity;
      property HistoryTickWidth: TTickWidth read FBoxWidth write SetBoxWidth;

      {
      Scale for calculating Tick Height
      }
      property MinValue: Integer read FMinValue write SetMinValue;
      property MaxValue: Integer read FMaxValue write SetMaxValue;

      {
      View Indicator in Design mode, used the random values for this
      }
      property RandomShowOnDesign: Boolean read FRandomShowOnDesign write SetRandomShowOnDesign;


      {inherited properties }
      property Color;
      property Align;
      property Enabled;
      property Width;
      property Height;
      property Visible;
      property ShowHint;
      property ParentShowHint;
      property BevelInner;
      property BevelOuter;
      property BevelWidth;
      property BorderStyle;
      property BorderWidth;



      property OnClick;
      property OnDblClick;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;

      property PopupMenu;

   end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('AP', [TOverloadIndicator]);
end;

{ TOverloadIndicator }

constructor TOverloadIndicator.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   Width := 200;
   FLastWidth := Width;
   Height := 50;
   FForegroundColor := clGreen;
   FBackgrnd := clWhite;
   FAutoQueryData := False;
   FMaxValue := 100;
   FMinValue := 0;
   FPosition := 50;
   FSpeedTPS := 2;
   FBoxWidth := 1;
   FHistoryCapacity := 20;
   FHistoryPointer := 1;
   FHistory := VarArrayCreate([1, FHistoryCapacity], varInteger);
   Randomize;
   FTimer := TTimer.Create(Self);
   FTimer.Interval := Round(1000 / FSpeedTPS);
   FTimer.OnTimer := OnTimer;
   FTimer.Enabled := True;

end;

destructor TOverloadIndicator.Destroy;
begin
   FTimer.Free;
   inherited;
end;

procedure TOverloadIndicator.ChangeBackgrndColor(const Value: TColor);
begin
   FBackgrnd := Value;
   Invalidate;
end;



procedure TOverloadIndicator.Paint;
var
   Rect: TRect;
   TopColor, BottomColor: TColor;

   procedure AdjustColors(Bevel: TPanelBevel);
   begin
      TopColor := clBtnHighlight;
      if Bevel = bvLowered then TopColor := clBtnShadow;
      BottomColor := clBtnShadow;
      if Bevel = bvLowered then BottomColor := clBtnHighlight;
   end;

begin
   //Paint Bevels
   Rect := GetClientRect;
   if BevelOuter <> bvNone then
      begin
         AdjustColors(BevelOuter);
         Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
      end;
   Frame3D(Canvas, Rect, Color, Color, BorderWidth);
   if BevelInner <> bvNone then
      begin
         AdjustColors(BevelInner);
         Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
      end;
   //Check for size changed
   if (FLastWidth <> Width) or (FLastBevelWidth <> BevelWidth) or (FLastBorderWidth <> BorderWidth) then
      begin
         //Recalc HistoryCapacity !!!
         SetBoxWidth(HistoryTickWidth);
         FLastWidth := Width;
         FLastBevelWidth := BevelWidth;
         FLastBorderWidth := BorderWidth;
      end;

   HistoryRepaint;
end;

procedure TOverloadIndicator.RePaint;
begin
   if not visible then Exit;
   Paint;
end;

procedure TOverloadIndicator.SetActive(const Value: Boolean);
begin
   FAutoQueryData := Value;
   invalidate;
end;

procedure TOverloadIndicator.SetHistoryCapacity(const Value: Integer);
begin
   FHistory := Unassigned;
   FHistory := VarArrayCreate([1, Value], varInteger);
   FHistoryCapacity := Value;
   FHistoryPointer := 1;
   RecalcBoxWidth;
   Invalidate;
end;

procedure TOverloadIndicator.SetMaxValue(const Value: Integer);
begin
   FMaxValue := Value;
   Invalidate;
end;

procedure TOverloadIndicator.SetMinValue(const Value: Integer);
begin
   FMinValue := Value;
   Invalidate;
end;

procedure TOverloadIndicator.SetOnDataNeed(const Value: TNeedDataEvent);
begin
   FOnDataNeed := Value;
end;

procedure TOverloadIndicator.SetPosition(const Value: Integer);
begin
   FPosition := Value;
   NewEvent(FPosition);
end;


procedure TOverloadIndicator.OnTimer(Sender: TObject);
var
   NewPos: Integer;
begin
   if ((csDesigning in ComponentState) and FRandomShowOnDesign) then
      begin
         NewEvent(Random(FMaxValue - FMinValue - 1) + 1);
      end
   else
      if (FAutoQueryData) and (Assigned(FOnDataNeed)) then
         begin
            NewPos := 0;
            FOnDataNeed(Sender, NewPos);
            NewEvent(NewPos);
         end;
end;

procedure TOverloadIndicator.SetSpeedTPS(
   const Value: TOverloadIndicatorSpeed);
begin
   FSpeedTPS := Value;
   FTimer.Interval := Round(1000 / FSpeedTPS);
end;

procedure TOverloadIndicator.NewEvent(NewPosition: Integer);
begin
   FHistory[FHistoryPointer] := NewPosition;
   if FHistoryPointer = FHistoryCapacity then
      FHistoryPointer := 1
   else
      inc(FHistoryPointer);
   FPosition := NewPosition;
   HistoryRepaint;
end;



procedure TOverloadIndicator.RecalcBoxWidth;
var
   BevelsWidth: Integer; //The Bevels Space
begin
   BevelsWidth := BevelWidth * 2 + BorderWidth;
   FBoxWidth := Round((ClientRect.Right - 2 * BevelsWidth) / FHistoryCapacity);
end;

procedure TOverloadIndicator.HistoryRepaint;
var
   CurrPointer: Integer;
   i: Integer;
   tmpBitmap: TBitmap;
   DrawRect: TRect;
   BevelsWidth: Integer; //The Bevels Space
begin
   // Paint Indicator
   BevelsWidth := BevelWidth * 2 + BorderWidth;

   DrawRect := Rect(BevelsWidth, BevelsWidth, ClientRect.Right - BevelsWidth, ClientRect.Bottom - BevelsWidth);


   tmpBitmap := TBitmap.Create;
   tmpBitmap.Width := FBoxWidth * FHistoryCapacity;
   tmpBitmap.Height := DrawRect.Bottom;

   tmpBitmap.Canvas.Brush.Color := FBackgrnd;
   tmpBitmap.Canvas.Brush.style := bsSolid;
   tmpBitmap.Canvas.Fillrect(Rect(0, 0, tmpBitmap.Width, DrawRect.Bottom));
   if Enabled then
      begin
         CurrPointer := FHistoryPointer;
         i := 0;
         tmpBitmap.Canvas.Brush.Color := FForegroundColor;
         tmpBitmap.Canvas.Brush.style := bsSolid;

         while i <> FHistoryCapacity do
            begin
               tmpBitmap.Canvas.Fillrect(Rect(i * FBoxWidth, DrawRect.Bottom - Round(FHistory[CurrPointer] / (FMaxValue - FMinValue) * (DrawRect.Bottom)), (i + 1) * FBoxWidth, DrawRect.Bottom));
               inc(i);
               if CurrPointer = FHistoryCapacity then
                  CurrPointer := 1
               else
                  inc(CurrPointer);
            end;
      end;
   Canvas.CopyRect(DrawRect, tmpBitmap.Canvas, Rect(0, 0, tmpBitmap.Width, DrawRect.Bottom));
   tmpBitmap.Free;
end;


procedure TOverloadIndicator.SetRandomShowOnDesign(const Value: Boolean);
begin
   FRandomShowOnDesign := Value;
   invalidate;
end;

procedure TOverloadIndicator.SetForegroundColor(const Value: TColor);
begin
   FForegroundColor := Value;
   Invalidate;
end;

procedure TOverloadIndicator.SetBoxWidth(const Value: TTickWidth);
var
   BevelsWidth: Integer; //The Bevels Space
begin
   FBoxWidth := Value;
   FHistory := Unassigned;
   BevelsWidth := BevelWidth * 2 + BorderWidth;
   FHistoryCapacity := Round((ClientRect.Right - 2 * BevelsWidth) / FBoxWidth);
   FHistory := VarArrayCreate([1, FHistoryCapacity], varInteger);
   FHistoryPointer := 1;
   invalidate;
end;


procedure TOverloadIndicator.ClearHistory;
begin
   FHistory := Unassigned;
   FHistory := VarArrayCreate([1, FHistoryCapacity], varInteger);
end;

end.

